<form class="account_edit" action="/accounts" method="POST">
<ol>
<li>
<label for="role">Role</label>
<select name="role" id="role">
<option value="">Please select role</option>
</select>
</li>
</ol>
